function [sys,x0,str,ts] = lddif_s(t,x,u,flag,m,n,l,del,poc,kov,tv)
% LDDIF_S    Estimation algorithm LDDIF
%            with Bittanti's correction (P(t)=P(t)+d*I, d<0.01)
%            Input:
%	 	              [y(t), u(t)]
%            Output:
%		              [theta(t), id_kovar, e]
%

% Identification LDDIF by Kulhavy and Karny, Budapest 1984
% x(n*m)     parameter vector (theta)
% z(n)       data vector
% y(m)       output vector
% e(m)       prediction error vector
% tv         sample period
% poc(n*m)   initial values of the parameters
% kov        covariance matrix
% n          number of data
% l          exponential forgetting l < 1 (l = 0.985)
% del        delta by bittanti (del < 0.01)

switch flag,
	case 0,
		[sys,x0,str,ts]=mdlInitializeSizes(m,n,l,del,poc,kov,tv);
	case 2,
		sys=mdlUpdate(t,x,u,m,n,l,del);
	case 3,
		sys=mdlOutputs(t,x,u);
	case 9,
		sys=mdlTerminate(t,x,u);
	otherwise
		error(['Unhandled flag = ',num2str(flag)]);
end

function [sys,x0,str,ts]=mdlInitializeSizes(m,n,l,del,poc,kov,tv)
	sizes = simsizes;
	sizes.NumContStates  = 0;
	sizes.NumDiscStates  = m*n+n*n+m;
	sizes.NumOutputs     = m*n+n*n+m;
	sizes.NumInputs      = n+m;
	sizes.DirFeedthrough = 0;
	sizes.NumSampleTimes = 1;
	sys = simsizes(sizes);
   
   if length(poc)~=n*m
		disp('*   LDDIF warning: Incorrect number of parameters !!!');
		disp('    Changed to:');
		poc = 0.1:0.1:n*m*0.1
	end
	if length(kov)==1
		p = eye(n,n)*kov;
	elseif length(kov)==n*n
		p = zeros(n,n);
		p(:) = kov;
	else
		disp('*   LDDIF warning: Incorrect covariance matrix !!!');
		disp('    Changed to:');
		p = eye(n)*10e6    
	end
   
   x0  = [poc,p(:)',zeros(1,m)]';
	str = [];
	ts  = [tv 0];

function sys=mdlUpdate(t,x,u,m,n,l,del)
	% initialisation of vectors and matrices
	p = zeros(n,n);
	p1 = zeros(n,1);
	p2 = zeros(n,1);
	theta = zeros(n,m);

	% get parameters and covariance matrix from state vector
	p(:) = x(m*n+1:m*n+n*n);
	theta(:) = x(1:m*n);
   y = u(1:m);
	z = u(m+1:n+m);

	% calculation of the prediction error
	e = y-theta'*z;
      
   % P = L D L'
   % L - lower triagonal matrix with ones on the main diagonal
	% D - diagonal matrix
	D = diag(diag(p));
	L = tril(p,-1)+eye(n);

	% s0,s2,p1,p2 - help vectors
	s0 = L'*z;
	p1 = D*s0;
 	s2 = p1'*s0;
	p2(n) = D(n,n)*s0(n)^2;
	for i=n-1:-1:1
		p2(i) = p2(i+1)+D(i,i)*s0(i)^2;
	end       
      
	if s2<1.0e-06
		sys = [theta(:);p(:);e];
		return
   end
   
	% Variable exponential forgetting
	s3 = l*s2+l-1;
	if abs(s3)<.0001
		p1 = L*p1;
	else
		s3 = s2/s3;
		s1 = p2(n)+s3;
		p(n,n) = p(n,n)*s3/s1+del;
		for j=n-1:-1:1
			s5 = p(j,j)*s1;
			s4 = p1(j)/s5;
			s1 = p2(j)+s3;
			p(j,j) = s5/s1+del;
			for i=j+1:n
				s5 = p(i,j);
				p(i,j) = s5-s4*p1(i);
				p1(i) = p1(i)+s5*p1(j);
			end
		end
	end
	s0 = 1/(1+s2);
	theta = theta+p1*e'*s0;
	sys = [theta(:);p(:);e];
function sys=mdlOutputs(t,x,u)
	sys = x;
function sys=mdlTerminate(t,x,u)
	sys = [];